// Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
// SPDX-License-Identifier: Apache-2.0

import 'package:amplify_api/src/graphql/factories/model_queries_factory.dart';
import 'package:amplify_core/amplify_core.dart';

/// Static helpers to generate query `GraphQLRequest` instances from models generated by Amplify codegen.

class ModelQueries {
  ModelQueries._(); // only static methods here, prevent calling constructor

  /// Generates a request for a single instance of the `modelType` by `id`.
  ///
  /// ```dart
  /// final request = ModelQueries.get(Todo.classType, 'some-todo-id-123');
  /// ```
  static GraphQLRequest<T> get<T extends Model>(
    ModelType<T> modelType,
    String id, {
    String? apiName,
    APIAuthorizationType? authorizationMode,
    Map<String, String>? headers,
  }) {
    return ModelQueriesFactory.instance.get<T>(
      modelType,
      id,
      apiName: apiName,
      authorizationMode: authorizationMode,
      headers: headers,
    );
  }

  /// Generates a request for a list of model instances.
  ///
  /// ```dart
  /// final request = ModelQueries.list(Todo.classType);
  /// ```
  /// or filter the query with some conditions:
  /// ```dart
  /// final request = ModelQueries.list(Todo.classType, where: Todo.NAME.contains('walk the dog'));
  /// ```
  static GraphQLRequest<PaginatedResult<T>> list<T extends Model>(
    ModelType<T> modelType, {
    int? limit,
    QueryPredicate? where,
    String? apiName,
    APIAuthorizationType? authorizationMode,
    Map<String, String>? headers,
  }) {
    return ModelQueriesFactory.instance.list<T>(
      modelType,
      limit: limit,
      where: where,
      apiName: apiName,
      authorizationMode: authorizationMode,
      headers: headers,
    );
  }
}
